open import Lib.Equality as Eq using (_==_ ; auto)
open import Lib.Maybe
open import Lib.Boolean
open import Lib.Proposition
open import Lib.List
open import Lib.Word
open import Lib.Word-Lemmas
open import Lib.Word-Tactics

open Monoid-Equational
open Rewriting
open Associative
open InContext

open import Lib.MaybeEq
open import Lib.MaybeEq-Instances
open import CosetNF
open import Gate

open import Step5.Rel
open import Step5.Comm

module Step5.Order where

  order-step : Step-Function Gate Rel
  order-step (CCX0-gen :: CCX0-gen :: xs) = Just (xs , at-head (axiom ax-CCX0-CCX0=ε))
  order-step (CCX1-gen :: CCX1-gen :: xs) = Just (xs , at-head (axiom ax-CCX1-CCX1=ε))
  order-step (CCX2-gen :: CCX2-gen :: xs) = Just (xs , at-head (axiom ax-CCX2-CCX2=ε))
  order-step (CX01-gen :: CX01-gen :: xs) = Just (xs , at-head (axiom ax-CX01-CX01=ε))
  order-step (CX10-gen :: CX10-gen :: xs) = Just (xs , at-head (axiom ax-CX10-CX10=ε))
  order-step (CX12-gen :: CX12-gen :: xs) = Just (xs , at-head (axiom ax-CX12-CX12=ε))
  order-step (CX21-gen :: CX21-gen :: xs) = Just (xs , at-head (axiom ax-CX21-CX21=ε))
  order-step (CX02-gen :: CX02-gen :: xs) = Just (xs , at-head (axiom ax-CX02-CX02=ε))
  order-step (CX20-gen :: CX20-gen :: xs) = Just (xs , at-head (axiom ax-CX20-CX20=ε))
  order-step (X0-gen :: X0-gen :: xs) = Just (xs , at-head (axiom ax-X0-X0=ε))
  order-step (X1-gen :: X1-gen :: xs) = Just (xs , at-head (axiom ax-X1-X1=ε))
  order-step (X2-gen :: X2-gen :: xs) = Just (xs , at-head (axiom ax-X2-X2=ε))
  order-step (Swap01-gen :: Swap01-gen :: xs) = Just (xs , at-head (axiom ax-Swap01-Swap01=ε))
  order-step (Swap12-gen :: Swap12-gen :: xs) = Just (xs , at-head (axiom ax-Swap12-Swap12=ε))
  order-step (S0-gen :: S0-gen :: S0-gen :: S0-gen :: xs) = Just (xs , at-head (axiom ax-S0-S0-S0-S0=ε))
  order-step (S1-gen :: S1-gen :: S1-gen :: S1-gen :: xs) = Just (xs , at-head (axiom ax-S1-S1-S1-S1=ε))
  order-step (S2-gen :: S2-gen :: S2-gen :: S2-gen :: xs) = Just (xs , at-head (axiom ax-S2-S2-S2-S2=ε))
  order-step (CS01-gen :: CS01-gen :: CS01-gen :: CS01-gen :: xs) = Just (xs , at-head (axiom ax-CS01-CS01-CS01-CS01=ε))
  order-step (CS12-gen :: CS12-gen :: CS12-gen :: CS12-gen :: xs) = Just (xs , at-head (axiom ax-CS12-CS12-CS12-CS12=ε))
  order-step (CS02-gen :: CS02-gen :: CS02-gen :: CS02-gen :: xs) = Just (xs , at-head (axiom ax-CS02-CS02-CS02-CS02=ε))
  order-step (CCZ-gen :: CCZ-gen :: xs) = Just (xs , at-head (axiom ax-CCZ-CCZ=ε))
  order-step (iI-gen :: iI-gen :: iI-gen :: iI-gen :: xs) = Just (xs , at-head (axiom ax-iI-iI-iI-iI=ε))
  order-step (K0-gen :: K0-gen :: xs) = Just (iI-gen :: iI-gen :: iI-gen :: xs , at-head (axiom ax-K0-K0=iI-iI-iI))
  order-step _ = Nothing

  module Order = Rewriting.Step (step-cong order-step)

  nf-order : ListNF Rel
  nf-order = record { listnf = Order.multistep 1000 ; lemma-listnf = Order.lemma-multistep 1000 } ∘ nf-comm

  nf-order-rep : ListNF Rel
  nf-order-rep = rep 20 (record { listnf = Order.multistep 1000 ; lemma-listnf = Order.lemma-multistep 1000 } ∘ nf-comm)


  order-K0-step : Step-Function Gate Rel
  order-K0-step (K0-gen :: K0-gen :: xs) = Just (iI-gen :: iI-gen :: iI-gen :: xs , at-head (axiom ax-K0-K0=iI-iI-iI))
  order-K0-step _ = Nothing

  module Order-K0 = Rewriting.Step (step-cong order-K0-step)

  nf-order-K0 : ListNF Rel
  nf-order-K0 = record { listnf = Order-K0.multistep 1000 ; lemma-listnf = Order-K0.lemma-multistep 1000 } ∘ nf-comm

  nf-order-K0-rep : ListNF Rel
  nf-order-K0-rep = rep 20 (record { listnf = Order-K0.multistep 1000 ; lemma-listnf = Order-K0.lemma-multistep 1000 } ∘ nf-comm)

 
