open import Lib.Equality as Eq using (_==_ ; auto)
open import Lib.Maybe
open import Lib.Boolean
open import Lib.Proposition
open import Lib.List
open import Lib.Word
open import Lib.Word-Lemmas
open import Lib.Word-Tactics

open Monoid-Equational
open Rewriting
open Associative
open InContext

open import Lib.MaybeEq
open import Lib.MaybeEq-Instances
open import CosetNF
open import Gate

import Step6.Rel as L

open import Step7.Rel
open import Step7.MvI
open import Step7.Order
open import Step7.Basis-Change
open import Step7.PLemmas
open import Step7.Comm

module Step7.S7 where

  isS1 : Gate -> Boolean
  isS1 _ = false

  isS2 : Gate -> Boolean
  isS2 CCX0-gen = true
  isS2 x = isS1 x


  isS3 : Gate -> Boolean
  isS3 CCX1-gen = true
  isS3 x = isS2 x

  isS4 : Gate -> Boolean
  isS4 CCX2-gen = true
  isS4 x = isS3 x

  isS5 : Gate -> Boolean
  isS5 CX01-gen = true
  isS5 x = isS4 x

  isS6 : Gate -> Boolean
  isS6 CX10-gen = true
  isS6 x = isS5 x

  isS7 : Gate -> Boolean
  isS7 CX02-gen = true
  isS7 CX20-gen = true
  isS7 CX12-gen = true
  isS7 CX21-gen = true
  isS7 x = isS6 x

  isS8 : Gate -> Boolean
  isS8 X0-gen = true
  isS8 X1-gen = true
  isS8 X2-gen = true
  isS8 x = isS7 x

  s7-step : Step-Function Gate Rel
  s7-step (CCX0-gen :: CCX0-gen :: xs) = Just (xs , at-head (axiom ax-CCX0-CCX0=ε))
  s7-step (CCX1-gen :: CCX1-gen :: xs) = Just (xs , at-head (lemma-CCX1-CCX1=ε))
  s7-step (CCX2-gen :: CCX2-gen :: xs) = Just (xs , at-head (lemma-CCX2-CCX2=ε))
  s7-step (CCX0-gen :: CCX1-gen :: CCX0-gen :: xs) = Just (CCX1-gen :: CCX0-gen :: CCX1-gen :: xs , at-head (axiom ax-CCX0-CCX1-CCX0=CCX1-CCX0-CCX1))
  s7-step (CCX1-gen :: CCX2-gen :: CCX1-gen :: xs) = Just (CCX2-gen :: CCX1-gen :: CCX2-gen :: xs , at-head (lemma-CCX1-CCX2-CCX1=CCX2-CCX1-CCX2))
  s7-step (CCX0-gen :: CCX2-gen :: CCX0-gen :: xs) = Just (CCX2-gen :: CCX0-gen :: CCX2-gen :: xs , at-head (lemma-CCX0-CCX2-CCX0=CCX2-CCX0-CCX2))
  s7-step (CCX0-gen :: CCX2-gen :: CCX1-gen :: CCX2-gen :: xs) = Just (CCX2-gen :: CCX1-gen :: CCX2-gen :: CCX0-gen :: xs , at-head (axiom ax-CCX0-CCX2-CCX1-CCX2=CCX2-CCX1-CCX2-CCX0))
  s7-step (CCX1-gen :: CCX2-gen :: CCX0-gen :: CCX2-gen :: xs) = Just (CCX2-gen :: CCX0-gen :: CCX2-gen :: CCX1-gen :: xs , at-head (lemma-CCX1-CCX2-CCX0-CCX2=CCX2-CCX0-CCX2-CCX1))
  s7-step (CCX2-gen :: CCX1-gen :: CCX0-gen :: CCX1-gen :: xs) = Just (CCX1-gen :: CCX0-gen :: CCX1-gen :: CCX2-gen :: xs , at-head (lemma-CCX2-CCX1-CCX0-CCX1=CCX1-CCX0-CCX1-CCX2))
  s7-step (CX01-gen :: CX01-gen :: xs) = Just (xs , at-head (axiom ax-CX01-CX01=ε))
  s7-step (CX10-gen :: CX10-gen :: xs) = Just (xs , at-head (lemma-CX10-CX10=ε))
  s7-step (CX12-gen :: CX12-gen :: xs) = Just (xs , at-head (lemma-CX12-CX12=ε))
  s7-step (CX21-gen :: CX21-gen :: xs) = Just (xs , at-head (lemma-CX21-CX21=ε))
  s7-step (CX02-gen :: CX02-gen :: xs) = Just (xs , at-head (lemma-CX02-CX02=ε))
  s7-step (CX20-gen :: CX20-gen :: xs) = Just (xs , at-head (lemma-CX20-CX20=ε))
  s7-step (CCX2-gen :: CX12-gen :: xs) = Just (CX12-gen :: CCX2-gen :: xs , at-head (lemma-CCX2-CX12=CX12-CCX2))
  s7-step (CCX2-gen :: CX02-gen :: xs) = Just (CX02-gen :: CCX2-gen :: xs , at-head (lemma-CCX2-CX02=CX02-CCX2))
  s7-step (CCX1-gen :: CX21-gen :: xs) = Just (CX21-gen :: CCX1-gen :: xs , at-head (lemma-CCX1-CX21=CX21-CCX1))
  s7-step (CCX1-gen :: CX01-gen :: xs) = Just (CX01-gen :: CCX1-gen :: xs , at-head (lemma-CCX1-CX01=CX01-CCX1))
  s7-step (CCX0-gen :: CX20-gen :: xs) = Just (CX20-gen :: CCX0-gen :: xs , at-head (lemma-CCX0-CX20=CX20-CCX0))
  s7-step (CCX0-gen :: CX10-gen :: xs) = Just (CX10-gen :: CCX0-gen :: xs , at-head (axiom ax-CCX0-CX10=CX10-CCX0))
  s7-step (CCX2-gen :: CX01-gen :: xs) = Just (CX01-gen :: CX02-gen :: CCX2-gen :: xs , at-head (axiom ax-CCX2-CX01=CX01-CX02-CCX2))
  s7-step (CCX2-gen :: CX10-gen :: xs) = Just (CX10-gen :: CX12-gen :: CCX2-gen :: xs , at-head (lemma-CCX2-CX10=CX10-CX12-CCX2))
  s7-step (CCX1-gen :: CX02-gen :: xs) = Just (CX02-gen :: CX01-gen :: CCX1-gen :: xs , at-head (lemma-CCX1-CX02=CX02-CX01-CCX1))
  s7-step (CCX1-gen :: CX20-gen :: xs) = Just (CX20-gen :: CX21-gen :: CCX1-gen :: xs , at-head (lemma-CCX1-CX20=CX20-CX21-CCX1))
  s7-step (CCX0-gen :: CX12-gen :: xs) = Just (CX12-gen :: CX10-gen :: CCX0-gen :: xs , at-head (lemma-CCX0-CX12=CX12-CX10-CCX0))
  s7-step (CCX0-gen :: CX21-gen :: xs) = Just (CX21-gen :: CX20-gen :: CCX0-gen :: xs , at-head (lemma-CCX0-CX21=CX21-CX20-CCX0))
  s7-step (CCX0-gen :: CX01-gen :: xs) = Just (CX01-gen :: CCX1-gen :: CCX0-gen :: CCX1-gen :: xs , at-head (axiom ax-CCX0-CX01=CX01-CCX1-CCX0-CCX1))
  s7-step (CCX1-gen :: CX10-gen :: xs) = Just (CX10-gen :: CCX0-gen :: CCX1-gen :: CCX0-gen :: xs , at-head (lemma-CCX1-CX10=CX10-CCX0-CCX1-CCX0))
  s7-step (CCX2-gen :: CX21-gen :: xs) = Just (CX21-gen :: CCX1-gen :: CCX2-gen :: CCX1-gen :: xs , at-head (lemma-CCX2-CX21=CX21-CCX1-CCX2-CCX1))
  s7-step (CCX2-gen :: CX20-gen :: CX02-gen :: xs) = Just (CX20-gen :: CX02-gen :: CCX0-gen :: xs , at-head (lemma-CCX2-CX20-CX02=CX20-CX02-CCX0))
  s7-step (CX10-gen :: CX01-gen :: CX10-gen :: xs) = Just (CX01-gen :: CX10-gen :: CX01-gen :: xs , at-head (lemma-CX10-CX01-CX10=CX01-CX10-CX01))
  s7-step (CX20-gen :: CX02-gen :: CX20-gen :: xs) = Just (CX02-gen :: CX20-gen :: CX02-gen :: xs , at-head (lemma-CX20-CX02-CX20=CX02-CX20-CX02))
  s7-step (CX21-gen :: CX12-gen :: CX21-gen :: xs) = Just (CX12-gen :: CX21-gen :: CX12-gen :: xs , at-head (lemma-CX21-CX12-CX21=CX12-CX21-CX12))
  s7-step (CX01-gen :: CX02-gen :: xs) = Just (CX02-gen :: CX01-gen :: xs , at-head (axiom ax-CX01-CX02=CX02-CX01))
  s7-step (CX01-gen :: CX21-gen :: xs) = Just (CX21-gen :: CX01-gen :: xs , at-head (axiom ax-CX01-CX21=CX21-CX01))
  s7-step (CX10-gen :: CX12-gen :: xs) = Just (CX12-gen :: CX10-gen :: xs , at-head (lemma-CX10-CX12=CX12-CX10))
  s7-step (CX10-gen :: CX20-gen :: xs) = Just (CX20-gen :: CX10-gen :: xs , at-head (lemma-CX10-CX20=CX20-CX10))
  s7-step (CX12-gen :: CX02-gen :: xs) = Just (CX02-gen :: CX12-gen :: xs , at-head (lemma-CX12-CX02=CX02-CX12))
  s7-step (CX21-gen :: CX20-gen :: xs) = Just (CX20-gen :: CX21-gen :: xs , at-head (lemma-CX21-CX20=CX20-CX21))
  s7-step (CX20-gen :: CX10-gen :: CX21-gen :: xs) = Just (CX21-gen :: CX10-gen :: xs , at-head (lemma-CX20-CX10-CX21=CX21-CX10))
  s7-step (CX20-gen :: CX21-gen :: CX10-gen :: xs) = Just (CX10-gen :: CX21-gen :: xs , at-head (lemma-CX20-CX21-CX10=CX10-CX21))
  s7-step (CX21-gen :: CX01-gen :: CX20-gen :: xs) = Just (CX20-gen :: CX01-gen :: xs , at-head (lemma-CX21-CX01-CX20=CX20-CX01))
  s7-step (CX21-gen :: CX02-gen :: CX01-gen :: xs) = Just (CX02-gen :: CX21-gen :: xs , at-head (lemma-CX21-CX02-CX01=CX02-CX21))
  s7-step (CX02-gen :: CX12-gen :: CX10-gen :: xs) = Just (CX10-gen :: CX02-gen :: xs , at-head (lemma-CX02-CX12-CX10=CX10-CX02))
  s7-step (CX01-gen :: CX20-gen :: CX21-gen :: xs) = Just (CX20-gen :: CX01-gen :: xs , at-head (lemma-CX01-CX20-CX21=CX20-CX01))
  s7-step (CX12-gen :: CX20-gen :: CX10-gen :: xs) = Just (CX20-gen :: CX12-gen :: xs , at-head (lemma-CX12-CX20-CX10=CX20-CX12))
  s7-step (CX12-gen :: CX10-gen :: CX02-gen :: xs) = Just (CX02-gen :: CX10-gen :: xs , at-head (lemma-CX12-CX10-CX02=CX02-CX10))
  s7-step (CX02-gen :: CX12-gen :: CX01-gen :: xs) = Just (CX01-gen :: CX12-gen :: xs , at-head (lemma-CX02-CX12-CX01=CX01-CX12))
  s7-step (CX02-gen :: CX21-gen :: CX01-gen :: xs) = Just (CX21-gen :: CX02-gen :: xs , at-head (lemma-CX02-CX21-CX01=CX21-CX02))
  s7-step (CX02-gen :: CX01-gen :: CX12-gen :: xs) = Just (CX12-gen :: CX01-gen :: xs , at-head (lemma-CX02-CX01-CX12=CX12-CX01))
  s7-step (CX10-gen :: CX02-gen :: CX12-gen :: xs) = Just (CX02-gen :: CX10-gen :: xs , at-head (lemma-CX10-CX02-CX12=CX02-CX10))
  s7-step (CX01-gen :: CX20-gen :: xs) = Just (CX20-gen :: CX21-gen :: CX01-gen :: xs , at-head (lemma-CX01-CX20=CX20-CX21-CX01))
  s7-step (CX01-gen :: CX10-gen :: CX02-gen :: CX10-gen :: CX01-gen :: xs) = Just (CX12-gen :: xs , at-head (lemma-CX01-CX10-CX02-CX10-CX01=CX12))
  s7-step (CX02-gen :: CX20-gen :: CX01-gen :: CX20-gen :: CX02-gen :: xs) = Just (CX21-gen :: xs , at-head (lemma-CX02-CX20-CX01-CX20-CX02=CX21))
  s7-step (CX02-gen :: CX20-gen :: CX12-gen :: CX20-gen :: CX02-gen :: xs) = Just (CX10-gen :: xs , at-head (lemma-CX02-CX20-CX12-CX20-CX02=CX10))
  s7-step (CX12-gen :: CX21-gen :: CX10-gen :: CX21-gen :: CX12-gen :: xs) = Just (CX20-gen :: xs , at-head (lemma-CX12-CX21-CX10-CX21-CX12=CX20))
  s7-step (CX12-gen :: CX21-gen :: CX02-gen :: CX21-gen :: CX12-gen :: xs) = Just (CX01-gen :: xs , at-head (lemma-CX12-CX21-CX02-CX21-CX12=CX01))
  s7-step (CCX0-gen :: CX02-gen :: CCX2-gen :: xs) = Just (CX02-gen :: CCX2-gen :: CCX0-gen :: xs , at-head (lemma-CCX0-CX02-CCX2=CX02-CCX2-CCX0))
  s7-step (CCX2-gen :: CX20-gen :: CCX0-gen :: xs) = Just (CX20-gen :: CCX0-gen :: CCX2-gen :: xs , at-head (lemma-CCX2-CX20-CCX0=CX20-CCX0-CCX2))
  s7-step (CCX1-gen :: CX12-gen :: CCX2-gen :: xs) = Just (CX12-gen :: CCX2-gen :: CCX1-gen :: xs , at-head (lemma-CCX1-CX12-CCX2=CX12-CCX2-CCX1))
  s7-step (CX21-gen :: CX02-gen :: CX20-gen :: CX02-gen :: xs) = Just (CX02-gen :: CX20-gen :: CX02-gen :: CX01-gen :: xs , at-head (lemma-CX21-CX02-CX20-CX02=CX02-CX20-CX02-CX01))
  s7-step _ = Nothing

  module S7 = Rewriting.Step (step-cong (swap-step then s7-step))
